Package: BioNERO
Type: Package
Title: Biological Network Reconstruction Omnibus
Version: 1.21.0
Authors@R: c(person("Fabricio", "Almeida-Silva", 
                    email="fabricio_almeidasilva@hotmail.com", 
                    role=c("cre", "aut"), 
                    comment = c(ORCID = "0000-0002-5314-2964")),
            person("Thiago", "Venancio", 
            role="aut", 
            comment = c(ORCID = "0000-0002-2215-8082")))
Description: BioNERO aims to integrate all aspects of biological network inference
        in a single package, including data preprocessing, exploratory analyses,
        network inference, and analyses for biological interpretations. BioNERO
        can be used to infer gene coexpression networks (GCNs) and gene 
        regulatory networks (GRNs) from gene expression data. Additionally,
        it can be used to explore topological properties of protein-protein 
        interaction (PPI) networks. GCN inference relies on the popular WGCNA 
        algorithm. GRN inference is based on the "wisdom of the crowds" principle,
        which consists in inferring GRNs with multiple algorithms (here, CLR,
        GENIE3 and ARACNE) and calculating the average rank for each interaction
        pair. As all steps of network analyses are included in this package,
        BioNERO makes users avoid having to learn the syntaxes of several 
        packages and how to communicate between them. Finally, users can also 
        identify consensus modules across independent expression sets and
        calculate intra and interspecies module preservation statistics 
        between different networks.
Depends: R (>= 4.1)
License: GPL-3
Encoding: UTF-8
LazyData: false
URL: https://github.com/almeidasilvaf/BioNERO
BugReports: https://github.com/almeidasilvaf/BioNERO/issues
biocViews: Software, GeneExpression, GeneRegulation, SystemsBiology,
        GraphAndNetwork, Preprocessing, Network, NetworkInference
Imports: WGCNA, dynamicTreeCut, ggdendro, matrixStats, sva,
        RColorBrewer, ComplexHeatmap, ggplot2, rlang, ggrepel,
        patchwork, reshape2, igraph, ggnetwork, intergraph, NetRep,
        stats, grDevices, utils, methods, BiocParallel, minet, GENIE3,
        SummarizedExperiment
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), BiocStyle, DESeq2,
        networkD3, covr
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/BioNERO
git_branch: devel
git_last_commit: 1351094
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:35:59 UTC; biocbuild
Author: Fabricio Almeida-Silva [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5314-2964>),
  Thiago Venancio [aut] (ORCID: <https://orcid.org/0000-0002-2215-8082>)
Maintainer: Fabricio Almeida-Silva <fabricio_almeidasilva@hotmail.com>
