# Generated by roxygen2: do not edit by hand

export(PC_correction)
export(SFT_fit)
export(ZKfiltering)
export(check_SFT)
export(consensus_SFT_fit)
export(consensus_modules)
export(consensus_trait_cor)
export(cor2adj)
export(cormat_to_edgelist)
export(detect_communities)
export(dfs2one)
export(enrichment_analysis)
export(exp2cor)
export(exp2gcn)
export(exp2gcn_blockwise)
export(exp2grn)
export(exp_genes2orthogroups)
export(exp_preprocess)
export(filter_by_variance)
export(gene_significance)
export(get_HK)
export(get_edge_list)
export(get_hubs_gcn)
export(get_hubs_grn)
export(get_hubs_ppi)
export(get_neighbors)
export(grn_average_rank)
export(grn_combined)
export(grn_filter)
export(grn_infer)
export(is_singleton)
export(modPres_WGCNA)
export(modPres_netrep)
export(module_enrichment)
export(module_preservation)
export(module_stability)
export(module_trait_cor)
export(net_stats)
export(parse_orthofinder)
export(plot_PCA)
export(plot_dendro_and_colors)
export(plot_eigengene_network)
export(plot_expression_profile)
export(plot_gcn)
export(plot_gene_significance)
export(plot_grn)
export(plot_heatmap)
export(plot_module_trait_cor)
export(plot_ngenes_per_module)
export(plot_ppi)
export(q_normalize)
export(remove_nonexp)
export(replace_na)
import(intergraph)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(ComplexHeatmap,pheatmap)
importFrom(GENIE3,GENIE3)
importFrom(NetRep,modulePreservation)
importFrom(RColorBrewer,brewer.pal)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(WGCNA,TOMsimilarity)
importFrom(WGCNA,adjacency)
importFrom(WGCNA,adjacency.fromSimilarity)
importFrom(WGCNA,bicor)
importFrom(WGCNA,blockwiseModules)
importFrom(WGCNA,checkSets)
importFrom(WGCNA,consensusMEDissimilarity)
importFrom(WGCNA,corPvalueFisher)
importFrom(WGCNA,corPvalueStudent)
importFrom(WGCNA,fundamentalNetworkConcepts)
importFrom(WGCNA,intramodularConnectivity)
importFrom(WGCNA,labels2colors)
importFrom(WGCNA,matchLabels)
importFrom(WGCNA,mergeCloseModules)
importFrom(WGCNA,moduleEigengenes)
importFrom(WGCNA,multiSetMEs)
importFrom(WGCNA,pickSoftThreshold)
importFrom(WGCNA,pquantile)
importFrom(WGCNA,sampledBlockwiseModules)
importFrom(WGCNA,scaleFreeFitIndex)
importFrom(WGCNA,signedKME)
importFrom(WGCNA,standardColors)
importFrom(dynamicTreeCut,cutreeDynamic)
importFrom(dynamicTreeCut,cutreeDynamicTree)
importFrom(ggdendro,dendro_data)
importFrom(ggnetwork,geom_edges)
importFrom(ggnetwork,geom_nodelabel_repel)
importFrom(ggnetwork,geom_nodes)
importFrom(ggnetwork,geom_nodetext)
importFrom(ggnetwork,ggnetwork)
importFrom(ggnetwork,theme_blank)
importFrom(ggnetwork,unit)
importFrom(ggplot2,aes)
importFrom(ggplot2,arrow)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,colorRampPalette)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,betweenness)
importFrom(igraph,centr_degree)
importFrom(igraph,cliques)
importFrom(igraph,closeness)
importFrom(igraph,cluster_edge_betweenness)
importFrom(igraph,cluster_fast_greedy)
importFrom(igraph,cluster_infomap)
importFrom(igraph,cluster_label_prop)
importFrom(igraph,cluster_leading_eigen)
importFrom(igraph,cluster_louvain)
importFrom(igraph,cluster_spinglass)
importFrom(igraph,cluster_walktrap)
importFrom(igraph,degree)
importFrom(igraph,diameter)
importFrom(igraph,edge_density)
importFrom(igraph,fit_power_law)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,simplify)
importFrom(igraph,transitivity)
importFrom(igraph,vcount)
importFrom(matrixStats,rowMedians)
importFrom(methods,is)
importFrom(minet,aracne)
importFrom(minet,build.mim)
importFrom(minet,clr)
importFrom(patchwork,wrap_plots)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,prcomp)
importFrom(stats,qnorm)
importFrom(stats,qqplot)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(sva,num.sv)
importFrom(sva,sva_network)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,write.table)
