
The data set "Orthogroups.tsv" was downloaded from OrthoFinder's example results.

```{bash eval=FALSE}
wget https://bioinformatics.plants.ox.ac.uk/davidemms/public_data/Results_model_species.tar.gz
tar -zxvf Results_model_species.tar.gz
head -n 20 Results_Nov26/Orthogroups/Orthogroups.tsv > Orthogroups.tsv
```
